"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const winston_1 = __importDefault(require("winston"));
const Router_1 = require("../Router");
const Server_1 = require("../Server");
const Controller_1 = require("../Controller");
const ResponseManager_1 = require("../Response/ResponseManager");
const ModuleManager_1 = require("../ModuleManager");
const defaultConfig = __importStar(require("../../config"));
const debug_1 = __importDefault(require("../../debug"));
const Config_1 = require("../Config");
const debug = debug_1.default('application');
class Application {
    constructor(config) {
        this.config = new Config_1.Config({});
        this.config.merge(this.config, defaultConfig, config);
    }
    bootstrap(config) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger = winston_1.default.createLogger(config.of('logger'));
            this.router = new Router_1.Router(config.of('router'));
            this.server = new Server_1.Server(this, config.of('server'));
            this.controllerManager = new Controller_1.ControllerManager(config.of('controller'));
            this.responseManager = new ResponseManager_1.ResponseManager(config.of('response'));
            this.moduleManager = new ModuleManager_1.ModuleManager(this, config.of('modules'));
            yield this.server.initialize();
            yield this.moduleManager.initialize();
            return this;
        });
    }
    getLogger() {
        return this.logger;
    }
    getRouter() {
        return this.router;
    }
    getServer() {
        return this.server;
    }
    getControllerManager() {
        return this.controllerManager;
    }
    getResponseManager() {
        return this.responseManager;
    }
    getConfig(section) {
        return this.config.of(section);
    }
    launch() {
        return __awaiter(this, void 0, void 0, function* () {
            debug('Launching server');
            yield this.bootstrap(this.config);
            this.server.start();
            debug('Server ready');
            return this;
        });
    }
}
exports.Application = Application;
//# sourceMappingURL=Application.js.map