"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
class ControllerManager {
    constructor(config) {
        this.controllers = {};
        this.config = config;
        this.loadControllers(this.config.location);
    }
    static getControllerName(controller) {
        if (typeof controller === 'string') {
            return controller;
        }
        return controller.name;
    }
    loadControllers(controllerDirectory) {
        const controllers = fs_1.default.readdirSync(controllerDirectory)
            .filter((fileName) => !!fileName.match(/^(?!(index)).+\.js$/))
            .map((fileName) => fileName.replace(/\.js$/, ''))
            .map((controller) => {
            const Controller = require(path_1.default.resolve(controllerDirectory, controller));
            if (typeof Controller === 'function') {
                return Controller;
            }
            if (typeof Controller.default === 'function') {
                return Controller.default;
            }
            if (typeof Controller[controller] === 'function') {
                return Controller[controller];
            }
            throw new TypeError(`Unable to load controller "${controller}" due to missing constructable export.`);
        });
        this.registerControllers(controllers);
    }
    getController(controller) {
        return this.controllers[ControllerManager.getControllerName(controller)];
    }
    registerControllers(controllers) {
        controllers.forEach(Controller => this.registerController(Controller));
        return this;
    }
    registerController(Controller) {
        this.controllers[Controller.name] = new Controller;
        return this;
    }
}
exports.ControllerManager = ControllerManager;
//# sourceMappingURL=ControllerManager.js.map