"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class Response {
    constructor({ message, data, meta, statusCode }) {
        this.headers = {};
        this.data = data;
        this.meta = meta;
        this.message = message;
        this.statusCode = statusCode;
    }
    patchContext(ctx) {
        this.ctx = ctx;
        this.setStatusCode(this.statusCode).applyHeaders().apply();
    }
    setStatusCode(statusCode) {
        this.ctx.status = statusCode;
        return this;
    }
    getStatusCode() {
        return this.statusCode;
    }
    setBody(body) {
        this.ctx.body = body;
        return this;
    }
    setHeaders(headers) {
        this.headers = headers;
        return this;
    }
    addHeaders(headers) {
        Reflect.ownKeys(headers).forEach((header) => this.setHeader(header, headers[header]));
        return this;
    }
    setHeader(header, value) {
        this.headers[header] = value;
        return this;
    }
    appendHeader(header, value) {
        this.headers[header] = [].concat(this.headers[header], value);
    }
    removeHeader(header) {
        delete this.headers[header];
        return this;
    }
    applyHeaders() {
        Reflect.ownKeys(this.headers).forEach((header) => this.ctx.set(header, this.headers[header]));
        return this;
    }
}
exports.Response = Response;
//# sourceMappingURL=Response.js.map