"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const koa_1 = __importDefault(require("koa"));
const cors_1 = __importDefault(require("@koa/cors"));
const koa_bodyparser_1 = __importDefault(require("koa-bodyparser"));
const router_1 = __importDefault(require("../../middleware/router"));
const request_1 = __importDefault(require("../../middleware/request"));
const dispatch_1 = __importDefault(require("../../middleware/dispatch"));
exports.DEFAULT_PORT = 1991;
class Server {
    constructor(application, config = {}) {
        this.application = application;
        this.config = config;
        this.create();
    }
    create() {
        this.server = new koa_1.default();
        return this;
    }
    use(...middlewares) {
        middlewares.forEach(middleware => this.server.use(middleware));
        return this;
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            const config = this.config;
            yield config.bootstrap(this.server, this);
            if (config.cors.enabled) {
                this.use(cors_1.default(config.cors.options));
            }
            this.use(request_1.default(), koa_bodyparser_1.default(), router_1.default(this.application), dispatch_1.default(this.application));
        });
    }
    getServer() {
        return this.server;
    }
    start() {
        this.server.listen(this.config.port || exports.DEFAULT_PORT);
        return this;
    }
}
exports.Server = Server;
//# sourceMappingURL=Server.js.map