"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Library_1 = require("../Library");
exports.default = (app) => function dispatch(ctx, next) {
    return __awaiter(this, void 0, void 0, function* () {
        // Dispatch needs to be the last middleware to run.
        // This leaves space for module-land and user-land middleware to run first.
        yield next();
        if (ctx.state.response) {
            return;
        }
        const { controller, action, controllerName } = ctx.state.dispatch;
        const logger = app.getLogger();
        const serverError = app.getResponseManager().serverError();
        // Route found, controller found... but the action doesn't exist. Or isn't a method.
        if (typeof controller[action] !== 'function') {
            logger.error(`Action "${action}" not found on controller "${controllerName}" for request path "${ctx.path}".`);
            ctx.state.response = serverError.notImplemented();
            return;
        }
        let response;
        try {
            response = yield controller[action](ctx);
            if (!(response instanceof Library_1.Response)) {
                throw new Library_1.InvalidActionResultError([
                    `Action "${controllerName}.${action}" failed to produce a Response instance,`,
                    `instead got type "${typeof response}".`,
                    'Did you forget to add a return statement in front of your response?',
                ].join(' '));
            }
        }
        catch (error) {
            logger.error(error.message, error);
            response = serverError.internalServerError(null, null, { error });
        }
        ctx.state.response = response;
    });
};
//# sourceMappingURL=dispatch.js.map